import unittest2 as unittest
import json


from resty.tests.mocks import MockStateMachine, MockDocument
from resty.documents import DocumentError


class TestJsonDocument(unittest.TestCase):

    def setUp(self):
        self.mock_sm = MockStateMachine()

        mock_d = MockDocument(
            meta={
                'type': 'T',
                'self': 'test://mock_resource/1/',
                'class': 'C1',
                'a': 'a', 'x': 'x',
            },
            content={'b': 'b', 'y': 'y'},
        )
        self.mock_sm.add_document(mock_d.self, mock_d)

        mock_d = MockDocument(
            meta={
                'type': 'T',
                'self': 'test://mock_resource/2/',
                'class': 'C2',
                'a': 'a', 'x': 'x',
            },
            content={'b': 'b', 'y': 'y'},
        )
        self.mock_sm.add_document(mock_d.self, mock_d)

        mock_d = MockDocument(
            meta={
                'type': 'T',
                'self': 'test://mock_resource/3/',
                'class': 'C3',
                'a': 'a', 'x': 'x',
            },
            content={'b': 'b', 'y': 'y'},
        )
        self.mock_sm.add_document(mock_d.self, mock_d)

        self.sentinel = object()
        self.mock_sm.add_document('test://a/b/', self.sentinel)
        self.mock_sm.add_document('test://test/', self.sentinel)
        self.mock_sm.add_document('test://test/1/', self.sentinel)
        self.mock_sm.add_document('test://test/2/', self.sentinel)

    def _get_target(self):
        from resty.documents import JsonDocument
        return JsonDocument

    def _make_one(self, meta={}, content={}):
        data = json.dumps(self._doc_repr(meta, content))
        return self._get_target()(self.mock_sm, data)

    def _doc_repr(self, meta={}, content={}):
        meta_copy = dict(('$%s' % key, value) for key, value in meta.items())
        meta_copy.update(content)
        return meta_copy

    def test_empty_json(self):
        self.assertRaises(ValueError, self._make_one)

    def test_required_fields(self):
        self.assertRaises(ValueError, self._make_one,
            content={'a': 'a', 'b': 'b'}
        )
        self.assertRaises(ValueError, self._make_one,
            meta={'type': 'type'}
        )
        self.assertRaises(ValueError, self._make_one,
            meta={'self': 'self'}
        )

    def test_minimal_doc(self):
        d = self._make_one(meta={'type': 'type', 'self': 'self'})
        self.assertEqual(d.type, 'type')
        self.assertEqual(d.self, 'self')
        self.assertRaises(AttributeError, getattr, d.meta, 'type')
        self.assertRaises(AttributeError, getattr, d.meta, 'self')

    def test_meta_attrs(self):
        d = self._make_one(
            meta={'type': 'type', 'self': 'self', 'a': 'a', 'b': 'b'}
        )
        self.assertEqual(d.meta.a, 'a')
        self.assertEqual(d.meta.b, 'b')

    def test_content_attrs(self):
        d = self._make_one(
            meta={'type': 'type', 'self': 'self'}, content={'a': 'a', 'b': 'b'}
        )
        self.assertEqual(d.content.a, 'a')
        self.assertEqual(d.content.b, 'b')

    def test_attr_errors(self):
        d = self._make_one(
            meta={'type': 'type', 'self': 'self', 'a': 'a'}, content={'b': 'b'}
        )
        self.assertEqual(d.meta.a, 'a')
        self.assertEqual(d.content.b, 'b')
        self.assertRaises(AttributeError, getattr, d.content, 'a')
        self.assertRaises(AttributeError, getattr, d.meta, 'b')

    def test_complex_structure(self):
        d = self._make_one(
            meta={'type': 'type', 'self': 'self'},
            content={'a': [1, 'a'], 'b': {'x': 'x', 'y': 1}}
        )
        self.assertEqual(d.content.a, [1, 'a'])
        self.assertEqual(d.content.b, {'x': 'x', 'y': 1})
        self.assertRaises(AttributeError, getattr, d.content.b, 'x')

    def test_class_meta(self):
        d = self._make_one(
            meta={'type': 'type', 'self': 'self', 'class': 'C'},
        )
        self.assertEqual(d.meta.class_, 'C')

    def test_filters(self):
        d = self._make_one(
            meta={
                'type': 'type',
                'self': 'self',
                'filters': {
                    'filter1': 'test://{placeholder1}/{placeholder2}/',
                    'filter2': 'test://test/',
                },
            },
        )

        f = d.filter('filter1', placeholder1='a', placeholder2='b')
        self.assertEqual(f, self.sentinel)

        f = d.filter('filter2')
        self.assertEqual(f, self.sentinel)

    def test_links(self):
        d = self._make_one(
            meta={
                'type': 'type', 'self': 'self',
                'links': [
                    self._doc_repr(
                        meta={
                            'type': 'T', 'self': 'test://mock_resource/1/',
                            'relationship': 'R1', 'class': 'C1',
                            'a': 'a',
                        },
                        content={'b': 'b'}
                    ),
                    self._doc_repr(
                        meta={
                            'type': 'T', 'self': 'test://mock_resource/2/',
                            'relationship': 'R2', 'class': 'C2',
                        }
                    ),
                    self._doc_repr(
                        meta={
                            'type': 'T', 'self': 'test://mock_resource/3/',
                            'relationship': 'R2', 'class': 'C3',
                        }
                    ),
                ],
            },
        )

        rel_doc = d.related('R1')
        self.assertEqual(rel_doc.meta.class_, 'C1')
        self.assertEqual(rel_doc.meta.a, 'a')
        self.assertEqual(rel_doc.meta.x, 'x')
        self.assertEqual(rel_doc.content.b, 'b')
        self.assertEqual(rel_doc.content.y, 'y')

        self.assertRaises(ValueError, d.related, 'R2')
        rel_doc = d.related('R2', 'C2')
        self.assertEqual(rel_doc.meta.class_, 'C2')
        rel_doc = d.related('R2', 'C3')
        self.assertEqual(rel_doc.meta.class_, 'C3')

    def test_services(self):
        d = self._make_one(
            meta={
                'type': 'type',
                'self': 'self',
                'services': {
                    'service1': self._doc_repr(
                        meta={'type': 'T', 'self': 'test://mock_resource/1/'}
                    ),
                    'service2': self._doc_repr(
                        meta={'type': 'T', 'self': 'test://mock_resource/2/'}
                    ),
                }
            },
        )

        serv_doc = d.service('service1')
        self.assertEqual(serv_doc.meta.a, 'a')
        self.assertEqual(serv_doc.content.b, 'b')
        self.assertEqual(serv_doc.meta.class_, 'C1')
        serv_doc = d.service('service2')
        self.assertEqual(serv_doc.meta.a, 'a')
        self.assertEqual(serv_doc.content.b, 'b')
        self.assertEqual(serv_doc.meta.class_, 'C2')
        self.assertRaises(ValueError, d.service, 'service3')

    def test_items(self):
        d = self._make_one(
            meta={
                'type': 'type',
                'self': 'self',
                'items': [
                    self._doc_repr(
                        meta={'type': 'T', 'self': 'test://mock_resource/1/'}
                    ),
                    self._doc_repr(
                        meta={'type': 'T', 'self': 'test://mock_resource/2/'}
                    ),
                    self._doc_repr(
                        meta={'type': 'T', 'self': 'test://mock_resource/3/'}
                    ),
                ],
            },
        )

        item_docs = list(d.items())
        self.assertEqual(item_docs[0].meta.class_, 'C1')
        self.assertEqual(item_docs[1].meta.class_, 'C2')
        self.assertEqual(item_docs[2].meta.class_, 'C3')

    def test_page(self):
        d = self._make_one(
            meta={
                'type': 'type',
                'self': 'self',
                'page_control': 'test://test/{page_num}/',
            },
        )

        self.assertEqual(d.page(1), self.sentinel)
        self.assertEqual(d.page(2), self.sentinel)

    def test_specialize(self):
        d = self._make_one(meta={'type': 'T', 'self': 'self'})
        self.assertEqual(d.specialize(), 'specialized T')

    def test_error(self):
        d = self._make_one(meta={'type': 'T', 'self': 'self'})

        self.assertRaises(DocumentError, d.related, 'R')
        self.assertRaises(DocumentError, d.service, 'S')
        self.assertRaises(DocumentError, d.items)
        self.assertRaises(DocumentError, d.page, 1)
        self.assertRaises(DocumentError, d.filter, 'F')


class TestLazyDocument(unittest.TestCase):

    def setUp(self):
        self.mock_sm = MockStateMachine()

        self.registered_d = MockDocument(
            meta={
                'type': 'T',
                'self': 'test://mock_resource/',
                'class': 'C1',
                'a': 'a', 'x': 'x',
            },
            content={'b': 'b', 'y': 'y'},
        )
        self.mock_sm.add_document(self.registered_d.self, self.registered_d)

        self.d = MockDocument(
            meta={
                'type': 'T', 'self': 'test://mock_resource/',
                'a': 'aa',
            },
            content={'b': 'bb'}
        )

    def _get_target(self):
        from resty.documents import LazyDocument
        return LazyDocument

    def _make_one(self, doc):
        return self._get_target()(self.mock_sm, doc)

    def test_fast_accesss(self):
        ld = self._make_one(self.d)

        self.assertEqual(ld.meta.a, 'aa')
        self.assertEqual(ld.content.b, 'bb')

    def test_slow_accesss(self):
        ld = self._make_one(self.d)

        self.assertEqual(ld.meta.x, 'x')
        self.assertEqual(ld.content.y, 'y')

    def test_reload(self):
        ld = self._make_one(self.d)

        ld.meta.x  # Trigger data reload
        self.assertEqual(ld.meta.a, 'a')
        self.assertEqual(ld.content.b, 'b')

    def test_attr_error(self):
        ld = self._make_one(self.d)

        self.assertRaises(AttributeError, getattr, ld.meta, 'not_existing')
        self.assertRaises(AttributeError, getattr, ld.content, 'not_existing')

    def test_deferred_related(self):
        sentinel = object()
        self.registered_d.add_related('R', sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.related('R'), sentinel)

    def test_deferred_service(self):
        sentinel = object()
        self.registered_d.add_service('S', sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.service('S'), sentinel)

    def test_deferred_items(self):
        sentinel = object()
        self.registered_d.add_item(sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.items(), [sentinel])

    def test_deferred_page(self):
        sentinel = object()
        self.registered_d.add_page(1, sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.page(1), sentinel)

    def test_deferred_filter(self):
        sentinel = object()
        self.registered_d.add_filter('F', sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.filter('F'), sentinel)

    def test_fast_related(self):
        sentinel = object()
        self.d.add_related('R', sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.related('R'), sentinel)

    def test_fast_service(self):
        sentinel = object()
        self.d.add_service('S', sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.service('S'), sentinel)

    def test_fast_items(self):
        sentinel = object()
        self.d.add_item(sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.items(), [sentinel])

    def test_fast_page(self):
        sentinel = object()
        self.d.add_page(1, sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.page(1), sentinel)

    def test_fast_filter(self):
        sentinel = object()
        self.registered_d.add_filter('F', sentinel)
        d = self._make_one(self.d)

        self.assertEqual(d.filter('F'), sentinel)



import copy
import json


class DocumentError(RuntimeError):
    pass


class Properties(object):
    def __init__(self, data, prefix=''):
        self.data = data
        self.prefix = prefix

    def __getattribute__(self, name):
        try:
            return object.__getattribute__(self, name)
        except AttributeError:
            if name == 'class_':
                prefixed = self.prefix + name[:-1]
            else:
                prefixed = self.prefix + name

            if prefixed in self.data:
                object.__setattr__(self, name, self.data[prefixed])
            return object.__getattribute__(self, name)

    def __setstate__(self, d):
        self.__dict__.update(d)


class JsonDocument(object):
    def __init__(self, state_machine, data):
        data = json.loads(data)
        self._sm = state_machine
        self._data = self._validated(data)

        self.type = self._data.pop('$type')
        self.self = self._data.pop('$self')

        self.meta = Properties(self._data, prefix='$')
        self.content = Properties(self._data)

    def _validated(self, data):
        if not data:
            raise ValueError

        required = ['$type', '$self']
        for r in required:
            if r not in data:
                raise ValueError
        return data

    def filter(self, name, **kwargs):
        if not hasattr(self.meta, 'filters'):
            raise DocumentError('No filters field.')
        if name not in self.meta.filters:
            raise ValueError

        filter_uri = self.meta.filters[name]
        for key, value in kwargs.iteritems():
            filter_uri = filter_uri.replace('{' + key + '}', value)
        return self._sm.load_document(filter_uri)

    def related(self, relation, class_=None):
        if not hasattr(self.meta, 'links'):
            raise DocumentError('No links field.')
        related = copy.deepcopy(self.meta.links)

        result = []
        for item in related:
            if relation == item.get('$relationship'):
                item.pop('$relationship')
                if class_ in [None, item.get('$class'), item.get('$elements')]:
                    result.append(item)

        if len(result) == 1:
            return self._from_subdoc(result.pop())

        raise ValueError

    def service(self, name):
        if not hasattr(self.meta, 'services'):
            raise DocumentError('No services field.')
        data = self.meta.services.get(name)
        if not data:
            raise ValueError
        return self._from_subdoc(data)

    def items(self):
        if not hasattr(self.meta, 'items'):
            raise DocumentError('No items field.')
        result = []
        for item in self.meta.items:
            result.append(self._from_subdoc(item))
        return result

    def specialize(self):
        return self._sm.specialize(self)

    def page(self, page):
        if not hasattr(self.meta, 'page_control'):
            raise DocumentError('No page_control field.')
        page_uri = self.meta.page_control.replace('{page_num}', str(page))
        return self._sm.load_document(page_uri)

    def _from_subdoc(self, subdoc):
        return self._sm.load_document(subdoc['$self'])


class LazyProperties(object):
    def __init__(self, loader):
        self._arg_loader = loader

    def __getattr__(self, name):
        return self._arg_loader(name)

    def __getstate__(self):
        return self.__dict__

    def __setstate__(self, d):
        self.__dict__.update(d)


class LazyDocument(object):
    def __init__(self, state_machine, doc):
        self._sm = state_machine

        self._original_doc = doc
        self._loaded_doc = None
        self._loaded = False

        self.type = doc.type
        self.self = doc.self

        self.meta = LazyProperties(self._get_meta)
        self.content = LazyProperties(self._get_content)

    def _get_meta(self, attr_name):
        if self._loaded:
            return getattr(self._loaded_doc.meta, attr_name)
        try:
            return getattr(self._original_doc.meta, attr_name)
        except AttributeError:
            self._loaded = True
            url = self._original_doc.self
            self._loaded_doc = self._sm.load_document(url)
            return self._get_meta(attr_name)

    def _get_content(self, attr_name):
        if self._loaded:
            return getattr(self._loaded_doc.content, attr_name)
        try:
            return getattr(self._original_doc.content, attr_name)
        except AttributeError:
            self._loaded = True
            url = self._original_doc.self
            self._loaded_doc = self._sm.load_document(url)
            return self._get_content(attr_name)

    def filter(self, name, **kwargs):
        return self._defer_method('filter', name, **kwargs)

    def related(self, relation, class_=None):
        return self._defer_method('related', relation, class_=class_)

    def service(self, name):
        return self._defer_method('service', name)

    def items(self):
        return self._defer_method('items')

    def specialize(self):
        return self._sm.specialize(self)

    def page(self, page):
        return self._defer_method('page', page)

    def _defer_method(self, method_name, *args, **kwargs):
        if self._loaded:
            return getattr(self._loaded_doc, method_name)(*args, **kwargs)
        try:
            return getattr(self._original_doc, method_name)(*args, **kwargs)
        except DocumentError:
            self._loaded = True
            self._loaded_doc = self._sm.load_document(self._original_doc.self)
            return self._defer_method(method_name, *args, **kwargs)


class LazyJsonDocument(JsonDocument):
    def _from_subdoc(self, subdoc):
        return LazyDocument(
            self._sm, LazyJsonDocument(
                self._sm, json.dumps(subdoc)
            )
        )



from resty.documents import DocumentError


class MockHttpLoader(object):

    def __init__(self):
        self._urls = {}

    def register(self, url, type, text):
        self._urls[url] = (type, text)

    def load(self, url):
        return self._urls[url]


class Attrs(object):
    pass


class MockDocument(object):

    def __init__(self, meta={}, content={}):

        self.type = meta.pop('type', 'T')
        self.self = meta.pop('self', 'test://mock_document')

        if 'class' in meta:
            meta['class_'] = meta.pop('class')

        self.meta = Attrs()
        self.content = Attrs()
        self.meta.__dict__.update(meta)
        self.content.__dict__.update(content)

        self._related = {}
        self._services = {}
        self._filters = {}
        self._pages = {}
        self._items = []

    def add_related(self, name, doc):
        self._related[name] = doc

    def add_service(self, name, doc):
        self._services[name] = doc

    def add_filter(self, name, doc):
        self._filters[name] = doc

    def add_item(self, doc):
        self._items.append(doc)

    def add_page(self, page_nr, doc):
        self._pages[page_nr] = doc

    def related(self, name, class_=None):
        if not self._related:
            raise DocumentError
        try:
            return self._related[name]
        except KeyError:
            raise ValueError

    def service(self, name):
        if not self._services:
            raise DocumentError
        try:
            return self._services[name]
        except KeyError:
            raise ValueError

    def filter(self, name, **kwargs):
        if not self._filters:
            raise DocumentError
        try:
            return self._filters[name]
        except KeyError:
            raise ValueError

    def specialize(self):
        return 'specialized %s' % self.type

    def items(self):
        if not self._items:
            raise DocumentError
        try:
            return self._items
        except KeyError:
            raise ValueError

    def page(self, nr):
        if not self._pages:
            raise DocumentError
        try:
            return self._pages[nr]
        except KeyError:
            raise ValueError


class MockStateMachine(object):

    def __init__(self):
        self._documents = {}

    def load_document(self, URI):
        return self._documents[URI]

    def add_document(self, state_id, document):
        self._documents[state_id] = document

    def specialize(self, doc):
        return 'specialized %s' % doc.type



import unittest2 as unittest


class TestPropertiesPickle(unittest.TestCase):

    def _get_target(self):
        from resty.documents import Properties
        return Properties

    def _make_one(self):
        return self._get_target()({})

    def test_pickable(self):
        import pickle
        pickle.loads(pickle.dumps(self._make_one()))


class TestLazyPropertiesPickle(TestPropertiesPickle):

    def _get_target(self):
        from resty.documents import LazyProperties
        return LazyProperties



import math


class Resource(object):
    def __init__(self, doc):
        self.doc = doc
        self.self = doc.self
        self.class_ = doc.meta.class_

    @property
    def edited(self):
        return self.doc.meta.edited

    @property
    def created(self):
        return self.doc.meta.created

    @property
    def hash(self):
        return self.doc.meta.hash

    @property
    def id(self):
        return self.doc.meta.id

    @property
    def content(self):
        return self.doc.content

    def related(self, relation, class_=None):
        rel = self.doc.related(relation, class_)
        return rel.specialize()


class Collection(object):
    def __init__(self, doc):
        self.doc = doc
        self.self = doc.self

    @property
    def hash(self):
        return self.doc.meta.hash

    @property
    def elements(self):
        return self.doc.meta.elements

    def filter(self, name, **kwargs):
        filtered = self.doc.filter(name, **kwargs)
        return filtered.specialize()

    def items(self):
        result = []
        if not hasattr(self.doc.meta, 'page'):
            for item in self.doc.items():
                result.append(item.specialize())
        else:
            total_nr_pages = math.ceil(
                float(self.doc.meta.items_count)
                / float(self.doc.meta.page_size)
            )
            for page_nr in range(1, int(total_nr_pages) + 1):
                page = self.doc.page(page_nr)
                for item in page.items():
                    result.append(item.specialize())
        return result


class Service(object):
    def __init__(self, doc):
        self.doc = doc
        self.self = doc.self

    @property
    def hash(self):
        return self.doc.meta.hash

    @property
    def content(self):
        return self.doc.content

    def service(self, name):
        service = self.doc.service(name)
        return service.specialize()



import unittest2 as unittest

from resty.tests.mocks import MockHttpLoader, MockDocument


class TestClient(unittest.TestCase):

    def _get_target(self):
        from resty import Client
        return Client

    def _make_one(self, loader):
        return self._get_target()(loader.load)

    def test_load_document(self):

        loader = MockHttpLoader()
        loader.register('test://load_document/1/', 'type1', 'text1')
        loader.register('test://load_document/2/', 'type2', 'text2')

        c = self._make_one(loader)

        class Mock(object):
            def __init__(self, client, text):
                self.client = client
                self.text = text

        class Mock1(Mock):
            pass

        class Mock2(Mock):
            pass

        c.register_document_parser('type1', Mock1)
        c.register_document_parser('type2', Mock2)

        d = c.load_document('test://load_document/1/')
        self.assertEqual(d.client, c)
        self.assertEqual(d.text, 'text1')
        self.assertTrue(isinstance(d, Mock1))

        d = c.load_document('test://load_document/2/')
        self.assertEqual(d.client, c)
        self.assertEqual(d.text, 'text2')
        self.assertTrue(isinstance(d, Mock2))

    def test_specialize(self):

        loader = MockHttpLoader()
        c = self._make_one(loader)

        class Mock(object):
            def __init__(self, doc):
                self.doc = doc

        class Mock1(Mock):
            pass

        class Mock2(Mock):
            pass

        c.register_document('type1', Mock1)
        c.register_document('type2', Mock2)

        doc1 = MockDocument(meta={'type': 'type1'})
        doc2 = MockDocument(meta={'type': 'type2'})

        s1 = c.specialize(doc1)
        self.assertTrue(isinstance(s1, Mock1))
        self.assertEqual(s1.doc, doc1)

        s2 = c.specialize(doc2)
        self.assertTrue(isinstance(s2, Mock2))
        self.assertEqual(s2.doc, doc2)



import os
import unittest2 as unittest
from resty import (
    Client, LazyJsonDocument, Service, Collection, Resource, JsonDocument
)


def disk_loader(uri):
    fixture_dir = os.path.join(os.path.dirname(__file__), 'mocks')
    fixture_path = os.path.join(fixture_dir, uri)
    mime_type = 'application/json'
    content = open(fixture_path).read()
    return (mime_type, content)


class TestClient(unittest.TestCase):

    def _get_target(self):
        client = Client(disk_loader)
        client.register_document_parser('application/json', LazyJsonDocument)
        client.register_document('application/vnd.test-service+json', Service)
        client.register_document(
            'application/vnd.test-collection+json', Collection
        )
        client.register_document(
            'application/vnd.test-resource+json', Resource
        )
        return client

    def test_available_cars(self):
        c = self._get_target()
        home = c.load('entrypoint.json')
        car_collection = home.service('cars')
        self.assertEqual(len(car_collection.items()), 3)

    def test_car_model(self):
        c = self._get_target()
        home = c.load('entrypoint.json')
        car_collection = home.service('cars')
        c = car_collection.items()[0]
        self.assertEqual(c.content.model, "Opel")

    def test_status_applications_by_customer1(self):
        c = self._get_target()
        home = c.load('entrypoint.json')
        customer_collection = home.service('customers')
        c1_resource = customer_collection.items()[0]
        related_app = c1_resource.related('children', 'Application')
        api_app_status = [r.content.status for r in related_app.items()]
        self.assertEqual(api_app_status, ['pending', 'approved'])

    def test_application5_history(self):
        c = self._get_target()
        app5_resource = c.load('applications/app5.json')
        customer_name = app5_resource.related('applied_by').content.name
        car_model = app5_resource.related('parent').content.model
        self.assertEqual([customer_name, car_model], ['Donald Bob', 'Opel'])

    def test_car2_rentals(self):
        c = self._get_target()
        car2_resource = c.load('cars/car2.json')
        car2_applications = car2_resource.related('children')
        api_applied_by = []
        for c in car2_applications.items():
            api_applied_by.append(c.related('applied_by').content.name)
        self.assertEqual(api_applied_by, ['John Smith', 'Donald Bob'])


class TestDumbClient(TestClient):

    def _get_target(self):
        dumb_client = Client(disk_loader)
        dumb_client.register_document_parser('application/json', JsonDocument)
        dumb_client.register_document(
            'application/vnd.test-service+json', Service
        )
        dumb_client.register_document(
            'application/vnd.test-collection+json', Collection
        )
        dumb_client.register_document(
            'application/vnd.test-resource+json', Resource
        )
        return dumb_client



import requests
from resty.documents import JsonDocument, LazyJsonDocument
from resty.types import Service, Collection, Resource


class Client(object):
    def __init__(self, loader):
        self.loader = loader
        self.register_parser = {}
        self.register = {}

    def load_document(self, uri):
        type_, text = self.loader(uri)
        doc = self.register_parser[type_]
        return doc(self, text)

    def register_document_parser(self, type_, class_):
        self.register_parser[type_] = class_

    def register_document(self, type_, class_):
        self.register[type_] = class_

    def specialize(self, class_):
        return self.register[class_.type](class_)

    def load(self, uri):
        d = self.load_document(uri)
        return d.specialize()


def http_loader(uri):
    response = requests.get(uri)
    return response.headers.get('content-type'), response.text


dumb_client = Client(http_loader)
dumb_client.register_document_parser('application/json', JsonDocument)
dumb_client.register_document('application/vnd.pbs-service+json', Service)
dumb_client.register_document(
    'application/vnd.pbs-collection+json', Collection
)
dumb_client.register_document('application/vnd.pbs-resource+json', Resource)

client = Client(http_loader)
client.register_document_parser('application/json', LazyJsonDocument)
client.register_document('application/vnd.pbs-service+json', Service)
client.register_document('application/vnd.pbs-collection+json', Collection)
client.register_document('application/vnd.pbs-resource+json', Resource)



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name='resty',
    version='0.1.1',
    description='PBS API consumer library',
    author='TPG CORE Services Team',
    author_email='tpg-pbs-coreservices@threepillarglobal.com',
    url='https://github.com/pbs/resty',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['requests'],
    setup_requires=['unittest2'],
    test_suite='unittest2.collector',
)



import unittest2 as unittest

from resty.tests.mocks import MockDocument


class TestResource(unittest.TestCase):

    def _get_target(self):
        from resty.types import Resource
        return Resource

    def _make_one(self, doc):
        return self._get_target()(doc)

    def test_known_meta(self):
        r = self._make_one(
            MockDocument(meta={
                'self': 'S',
                'hash': 'hash',
                'class': 'Class',
                'id': 'ID',
                'created': 'created',
                'edited': 'edited',
            })
        )

        self.assertEqual(r.self, 'S')
        self.assertEqual(r.hash, 'hash')
        self.assertEqual(r.class_, 'Class')
        self.assertEqual(r.id, 'ID')
        self.assertEqual(r.created, 'created')
        self.assertEqual(r.edited, 'edited')

    def test_content(self):
        r = self._make_one(
            MockDocument(
                meta={'class': 'C'},
                content={'a': 'a', 'b': 'b'}
            )
        )
        self.assertEqual(r.content.a, 'a')
        self.assertEqual(r.content.b, 'b')

    def test_attr_error(self):
        r = self._make_one(MockDocument(meta={'class': 'C'}))
        self.assertRaises(AttributeError, getattr, r.content, 'not_found')
        self.assertRaises(AttributeError, getattr, r, 'edited')

    def test_get_related(self):
        doc = MockDocument(
            meta={'type': 'T', 'self': 'test://document/', 'class': 'C'}
        )
        doc.add_related(
            'relationship1',
            MockDocument(meta={'type': 'R1', 'self': 'test://related'})
        )
        doc.add_related(
            'relationship2',
            MockDocument(meta={'type': 'R2', 'self': 'test://related'})
        )
        r = self._make_one(doc)
        self.assertEqual(r.related('relationship1'), 'specialized R1')
        self.assertEqual(r.related('relationship2'), 'specialized R2')


class TestPickledResoruce(TestResource):
    def _make_one(self, *args, **kwargs):
        one = super(TestPickledResoruce, self)._make_one(*args, **kwargs)
        import pickle
        return pickle.loads(pickle.dumps(one))


class TestService(unittest.TestCase):

    def _get_target(self):
        from resty.types import Service
        return Service

    def _make_one(self, doc):
        return self._get_target()(doc)

    def test_known_meta(self):
        r = self._make_one(MockDocument(meta={'self': 'S', 'hash': 'hash'}))

        self.assertEqual(r.self, 'S')
        self.assertEqual(r.hash, 'hash')

    def test_content(self):
        r = self._make_one(MockDocument(content={'a': 'a', 'b': 'b'}))
        self.assertEqual(r.content.a, 'a')
        self.assertEqual(r.content.b, 'b')

    def test_attr_error(self):
        r = self._make_one(MockDocument())
        self.assertRaises(AttributeError, getattr, r.content, 'not_found')
        self.assertRaises(AttributeError, getattr, r, 'edited')

    def test_get_related(self):
        doc = MockDocument(
            meta={'type': 'T', 'self': 'test://document/'}
        )
        doc.add_service('s1',
            MockDocument(meta={'type': 'R1', 'self': 'test://related'})
        )
        doc.add_service('s2',
            MockDocument(meta={'type': 'R2', 'self': 'test://related'})
        )
        r = self._make_one(doc)
        self.assertEqual(r.service('s1'), 'specialized R1')
        self.assertEqual(r.service('s2'), 'specialized R2')


class TestPickledService(TestService):
    def _make_one(self, *args, **kwargs):
        one = super(TestPickledService, self)._make_one(*args, **kwargs)
        import pickle
        return pickle.loads(pickle.dumps(one))


class TestCollection(unittest.TestCase):

    def _get_target(self):
        from resty.types import Collection
        return Collection

    def _make_one(self, doc):
        return self._get_target()(doc)

    def test_known_meta(self):
        r = self._make_one(MockDocument(meta={
            'self': 'S',
            'elements': 'ET',
            'hash': 'hash',
        }))

        self.assertEqual(r.self, 'S')
        self.assertEqual(r.elements, 'ET')
        self.assertEqual(r.hash, 'hash')

    def test_attr_error(self):
        r = self._make_one(MockDocument(meta={'elements': 'ET'}))
        self.assertRaises(AttributeError, getattr, r, 'hash')

    def test_filter(self):
        doc = MockDocument(meta={'elements': 'ET'})
        doc.add_filter('zipcode', MockDocument(meta={'type': 'ZIP'}))
        doc.add_filter('IP', MockDocument(meta={'type': 'IP'}))
        r = self._make_one(doc)
        self.assertEqual(r.filter('zipcode'), 'specialized ZIP')
        self.assertEqual(r.filter('IP'), 'specialized IP')
        self.assertRaises(ValueError, r.filter, 'err')

    def test_iter_simple(self):
        doc = MockDocument(meta={'elements': 'ET'})
        doc.add_item(MockDocument(meta={'type': 'T1'}))
        doc.add_item(MockDocument(meta={'type': 'T2'}))
        r = self._make_one(doc)
        items = list(r.items())
        self.assertEqual(items, ['specialized T1', 'specialized T2'])

    def test_iter_multi_page(self):

        page1 = MockDocument(meta={'elements': 'ET'})
        page1.add_item(MockDocument(meta={'type': 'T1'}))
        page1.add_item(MockDocument(meta={'type': 'T2'}))

        page2 = MockDocument(meta={'elements': 'ET'})
        page2.add_item(MockDocument(meta={'type': 'T3'}))
        page2.add_item(MockDocument(meta={'type': 'T4'}))

        page3 = MockDocument(meta={'elements': 'ET'})
        page3.add_item(MockDocument(meta={'type': 'T5'}))

        page1.meta.page_size = page2.meta.page_size = page3.meta.page_size = 2
        page1.meta.items_count = 5
        page2.meta.items_count = 5
        page3.meta.items_count = 5
        page1.meta.page, page2.meta.page, page3.meta.page = 1, 2, 3

        page1.add_page(1, page1)
        page1.add_page(2, page2)
        page1.add_page(3, page3)

        page2.add_page(1, page1)
        page2.add_page(2, page2)
        page2.add_page(3, page3)

        page3.add_page(1, page1)
        page3.add_page(2, page2)
        page3.add_page(3, page3)

        r = self._make_one(page1)
        items = list(r.items())
        self.assertEqual(items, [
            'specialized T1',
            'specialized T2',
            'specialized T3',
            'specialized T4',
            'specialized T5',
        ])


class TestPickledCollection(TestCollection):
    def _make_one(self, *args, **kwargs):
        one = super(TestPickledCollection, self)._make_one(*args, **kwargs)
        import pickle
        return pickle.loads(pickle.dumps(one))
